@@ -2,12 +2,12 @@ from flask import *  # noqa: Expose all flask objects as top level imports
 from flask import request
 from slack import WebClient as Slack
 
-from .flack import Flack
+from .flack import Slackify
 from .slack import ACK, OK, block_reply, reply_text, reply, respond, text_block
 from .tasks import async_task
 
 __all__ = [
-    'Flack',
+    'Slackify',
     'Slack',
     'request',
     'reply_text',
